import * as THREE from '../build/three.module.js';

import Stats from './jsm/libs/stats.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        renderer.toneMapping = THREE.ReinhardToneMapping;
        domWebGL.appendChild(renderer.domElement);

        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0xf0f0f0);
        this._scene = scene;

        this._setupCamera();
        this._setupLight();
        //this._setupPostprocessing();
        //this._setupInteractions();
        this._setupModel();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();

        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {}

    _setupPostprocessing() {}

    _setupModel() { 
        const colors = [ 0xff0000, 0x00ff00, 0x0000ff ];
        const geometry = new THREE.BoxGeometry(20, 20, 20);

        for(let i=0; i<500; i++) {
            const layer = (i % 3);
            const object = new THREE.Mesh(geometry, new THREE.MeshLambertMaterial({ color:colors[layer] }));

            object.position.x = Math.random() * 800 - 400;
            object.position.y = Math.random() * 800 - 400;
            object.position.z = Math.random() * 800 - 400;

            object.rotation.x = Math.random() * 2 * Math.PI;
            object.rotation.y = Math.random() * 2 * Math.PI;
            object.rotation.z = Math.random() * 2 * Math.PI;

            object.scale.x = Math.random() + 0.5;
            object.scale.y = Math.random() + 0.5;
            object.scale.z = Math.random() + 0.5;

            object.layers.set(layer);

            this._scene.add(object);
        }
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.domElement);
        this._stats = stats;

        const layers = {
            "toggle red": () => {
                this._camera.layers.toggle(0);
            },
            "toggle green": () => {
                this._camera.layers.toggle(1);
            },
            "toggle blue": () => {
                this._camera.layers.toggle(2);
            },
            "enable all": () => {
                this._camera.layers.enableAll();
            },
            "disable all": () => {
                this._camera.layers.disableAll();
            }
        };

        const gui = new GUI();
        gui.add(layers, "toggle red");
        gui.add(layers, "toggle green");
        gui.add(layers, "toggle blue");
        gui.add(layers, "enable all");
        gui.add(layers, "disable all");
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(100, window.innerWidth / window.innerHeight, 1, 10000);
        camera.layers.enable(0); // 기본으로 활성화됨
        camera.layers.enable(1);
        camera.layers.enable(2);

        this._scene.add(camera);

        this._camera = camera;
    }

    _setupControls() {}

    _setupLight() {
        const light = new THREE.PointLight(0xffffff, 1);
        light.layers.enable(0);
        light.layers.enable(1);
        light.layers.enable(2);

        this._camera.add(light);
    }

    update() {
        const t = performance.now();
        this._camera.position.x = 100 * Math.sin(THREE.MathUtils.degToRad(t) * 0.01);
        this._camera.position.y = 100 * Math.sin(THREE.MathUtils.degToRad(t) * 0.01);
        this._camera.position.z = 100 * Math.cos(THREE.MathUtils.degToRad(t) * 0.01);
        //this._camera.lookAt(this._scene.position);

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this._renderer.render(this._scene, this._camera);
        this.update();
    }

    resize() {
        const width = window.innerWidth;
        const height = window.innerHeight;

        this._camera.aspect = width / height;
        this._camera.updateProjectionMatrix();
        
        this._renderer.setSize(width, height);
    }
}

window.onload = function () {
    new App();
}